#include <stdio.h>
#include <time.h>

#include "list_rank.h"
#include "list_text.h"

#define		ASC	0x01
#define		SPA	0x02
#define 	ETR 	0x10
#define 	DOT	0x11
#define		UNK	0x12
#define		NUL	0x13
#define		TAB	0x14
#define		UN2	0x15
#define		EN2	0x16
#define		UN3	0x17

// Artemio Urbina Garcia 2001, MGS text dumper (for PSX/eng)

void printuse();
void extract(char *File, int start, int end, FILE *dump, int pbold);

int main(int argc, char *argv[])
{
        float value;
	FILE *dump;
	clock_t prin, fin;
	int pbold, i;
	l_rank ranks, trank;

        prin = clock();

	pbold = 0;
	if(argc < 2)
		printuse();

	for(i = 1; i < argc; i ++)
	{
		int len;
		char buffer[1024];

		len = strlen(argv[i]);
		if(len >= 1024)
		{
			printf("Argument too large\n");
			exit(1);
		}

		// yes TOO dirty, but fast =P
		if(!strcmp("stage.dir", argv[i]))
		{
			printf("Dir file, special case\n");
			pbold = 1;
		}
		else
			pbold = 0;

		ranks = new_rank();
		sprintf(buffer, "%s", argv[i]);
		buffer[len - 3] = 'r';
		buffer[len - 2] = 'n';
		buffer[len - 1] = 'k';
		ranks = read_ranks(buffer);

		buffer[len - 3] = 't';
		buffer[len - 2] = 'x';
		buffer[len - 1] = 't';
		if(!strcmp(buffer, argv[i]))
		{
			printf("Source equals destination\n");
			exit(1);
		}
		dump = fopen(buffer, "w");
		if(!dump)
		{
			printf("Could not open file %s\n", buffer);
			exit(1);
		}
		trank = ranks;

		printf("Extracting to %s ", buffer);
		fprintf(dump, "Metal Gear PSX Dumper\n");
		fprintf(dump, "By: Artemio Urbina (aurbina@junkerhq.net)\n");
		fprintf(dump, "\n");

		
		if(!ranks)
		{
			printf("No ranks file found... dumping all\n");
			extract(argv[i], 0, 0xfffffff, dump, pbold);
		}
		else
		{
			printf("Ranks file found... dumping\n");
			while(trank)
			{
				extract(argv[i], trank->lower,
					trank->upper, dump, pbold);
				trank = trank->next;
			}
			ranks = free_ranks(ranks);
		}
	
		fclose(dump);
	}
        fin = clock();
        value = (float)(fin - prin)/CLOCKS_PER_SEC;
	printf("Process time: %g seconds\n", value);
	return(0);
}

void printuse()
{
	printf("Use: extract <Src> FileName [-b]\n");
	printf("  AddrX must be hex values without 0x prefix\n");
	printf("  Ex: extract 1f34a 235fb script.sjs\n");
	printf("  -b sets the bold to <B>text</B> (optional)\n");
	exit(1);
}

void extract(char *File, int start, int end, FILE *dump, int pbold)
{
	int c, intext, rank;
	long pos, tmppos;
	FILE *file;
	l_rank	list;
	ascii_array array;
	l_text	ltext;

	pos = 0;
	intext = 0;
	c = 'A';
	ltext = NULL;
	list = new_rank();
	array = new_ascii();

	file = fopen(File, "rb");
	if(!file)
	{
		printf("Could not open file %s (Copy it from CD)\n", File);
		exit(1);
	}

	if(fseek(file, sizeof(char)*start, SEEK_SET) != 0)
	{
		fclose(dump);
		fclose(file);
		printf("Could not set start offset");
		exit(1);
	}
	list = read_ranks("mgsranks.rnk");
	tmppos = start;
	do
	{
		pos = tmppos;
		c = fgetc(file);
		pos ++;
		rank = InsideRank(c, list);
		if(rank)
		{
			if(!intext)
				array = free_ascii(array);
			switch(rank)
			{
				case NUL:
					if(!intext)
						break;
					intext = 0;
					if(array->csize > 2)
					{
						switch(array->array[array->csize-1])
						{
							case '!':
							case '?':
							case '.':
							case ',':
								ltext = insert_text(array, ltext);
								array = NULL;
								break;
							default:
								if(array->csize >= 8) // Smallest string in vox.dat Disc 1 (17)
								{
									ltext = insert_text(array, ltext);
									array = NULL;
								}
								break;
								
						}
					}
					if(array)
						array = free_ascii(array);
					break;
				case UNK:
				case UN2:
				case UN3:
					if(!pbold && !intext)
						break;
					c = fgetc(file);
					pos ++;
					switch(c)
					{
					case 0x7c:
						array = insert_ascii(ENTER, array);
						break;
					/*
					case 0x22:	// Color change in "Deepthroat"
						//array = insert_ascii(COLOR, array);
						//array = insert_ascii(c, array);
						break;
						*/
					case 0x01:
						array = insert_ascii(TRIANGLE, array);
						break;
					default:
						array = insert_ascii(ENTER, array);
						//array = insert_ascii(UNKNOWN, array);
						//array = insert_ascii(0x80, array);
						//array = insert_ascii(c, array);
					}
					break;
				case SPA: //Spanish character
					if(!intext)
					{
						intext = 1;
						array = setaddress(pos - 1, array);
					}
					c = fgetc(file);
					pos ++;
					switch(c)
					{
						case 0x08:
							array = insert_ascii((int)'', array);
							break;
						case 0x0f:
							array = insert_ascii((int)'', array);
							break;
						case 0x13:
							array = insert_ascii((int)'', array);
							break;
						case 0x18:
							array = insert_ascii((int)'', array);
							break;
						case 0x1d:
							array = insert_ascii((int)'', array);
							break;
						case 0x41:
							array = insert_ascii((int)'', array);
							break;
						case 0x42:
							array = insert_ascii((int)'', array);
							break;
						case 0x4a:	//Unknown...
							array = insert_ascii((int)' ', array);
							break;
						case 0x20:
							array = insert_ascii((int)'', array);
							break;
						case 0x17:
							array = insert_ascii((int)'', array);
							break;
						default:
							array = insert_ascii(SPANISH, array);
							array = insert_ascii(c, array);	
							break;
					}
					break;
				case ETR:	// Enter in string
					if(!intext)
						break;
					array = insert_ascii(ENTER, array);
					break;
				case EN2: // Special case, just a few lines about missiles (0x0d0a)
					break;
				case DOT:
					c = '.';
				case TAB:
				case ASC:
					if(!intext)
					{
						intext = 1;
						array = setaddress(pos - 1, array);
					}
					array = insert_ascii(c, array);
					break;
			}	
		}
		else
		{
			intext = 0;
			array = free_ascii(array);
		}
		tmppos = pos;
		if(!end)
		{
			pos = -1;
		}
	}
	while(c != EOF && pos <= end);
	
	print_texts(ltext, dump);
	free_texts(ltext);
	fclose(file);
	list = free_ranks(list);
}
